/****************************************************************************
 READ_RES.C   DEMO PROGRAM for retrieving 3048A result files.

 Author: Marc E. Brodeur

 COPYRIGHT (c) 1993, Hewlett-Packard Co.

 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>


#include "globals.h"         /* structures and some global variables */

/* This macro performs a disk operation (fread or fwrite) and checks for
   success.  If the required number of items were not read or written, the
   local variable 'no_error' is set to FALSE and the do-while loop is
   exited.
*/
#define CHECK_ERROR(function, req_nr)  \
  if (function != req_nr) {no_error = FALSE; break;}

BOOLEAN load_results (FILE *fil, int *ff1, int *ff2, int *rf1, int *rf2);
void generate_freq_ampl_pairs(int ff1, int ff2, int rf1, int rf2, char *output_file_name);
BOOLEAN unscale_trace_data(double i_ampl, double *dbc);
FILE *open_binary_file(char *b_file_name);
/*************************************************************************/

void main(int argc, char*argv[])
{
   int i;

   BOOLEAN no_err = FALSE;
   FILE *result_file;
   int ff1, ff2, rf1, rf2;
   char infile[12], outfile[12];

   clrscr();

   printf("Opening input file ...\n");

   if (argc < 3)
   {
     fprintf(stderr,"USAGE:\n       READ_RES.EXE [*.RES filename] [output filename]\n");
     exit (1);
   }

   strcpy(infile, argv[1]);

   if ( !((strstr(infile,".RES")) || (strstr(infile,".res"))))
   {

     result_file = open_binary_file(strcat(infile,".RES"));
   }
   else
   {
     result_file = open_binary_file(infile);
   }


   no_err = load_results(result_file, &ff1, &ff2, &rf1, &rf2);

   if (!no_err)
   {
     printf("Error encountered in reading file\n");
     exit(1);
   }

   strcpy(outfile,argv[2]);
   generate_freq_ampl_pairs(ff1, ff2, rf1, rf2, outfile);

   printf("File translation complete.  Have a nice day !!!\n");

   exit(0);
}



/****************************************************************************
 load_results() - load a HP3048A results (*.RES) file which was saved
		  as a binary file.

	  fil -> pointer to already open file pointer for results file
	 *ff1 -> 1st fft segment used.
	 *ff2 -> last fft segment used.
	 *rf1 -> 1st rf analyzer segment used.
	 *rf2 -> last rf analyzer segment used.

 Note:  This function uses the MACRO CHECK_ERROR which is defined at the top
	of this source file.
 ****************************************************************************/
BOOLEAN load_results (FILE *fil, int *ff1, int *ff2, int *rf1, int *rf2)
{
     BOOLEAN no_error = TRUE;   /* set false by CHECK_ERROR and then break */
     int j;

     printf("Loading input file data ... \n");



     do                         /* do-while to allow breaking out on error */
     {

	  CHECK_ERROR (fread (&Starttime, sizeof(TIMEDATE), 1, fil), 1);
	  CHECK_ERROR (fread (&Stoptime,  sizeof(TIMEDATE), 1, fil), 1);
	  CHECK_ERROR (fread (&Carrier,   sizeof(double), 1, fil), 1);
	  CHECK_ERROR (fread (Gtitle,     sizeof(char), 81, fil), 81);

	  CHECK_ERROR (fread (ff1, sizeof(int), 1, fil), 1);
	  CHECK_ERROR (fread (ff2, sizeof(int), 1, fil), 1);
	  CHECK_ERROR (fread (rf1, sizeof(int), 1, fil), 1);
	  CHECK_ERROR (fread (rf2, sizeof(int), 1, fil), 1);

	  if (*ff1 >= 0 && *ff2 >= 0)
	       CHECK_ERROR (fread (&(Fft_plot[*ff1][0]), sizeof(int),
		 401 * (*ff2 - *ff1 + 1), fil), 401 * (*ff2 - *ff1 + 1));


	  if (*rf1 >= 0 && *rf2 >= 0)
	       CHECK_ERROR (fread (&(Hf_plot[*rf1][0]), sizeof(int),
		 1001 * (*rf2 - *rf1 + 1), fil), 1001 * (*rf2 - *rf1 + 1));

          CHECK_ERROR (fread (Fft_tags, sizeof(double), FFTSEGS * 4, fil),
                       FFTSEGS * 4);
          CHECK_ERROR (fread (Hf_tags,  sizeof(double), HFSEGS * 5, fil),
                       HFSEGS * 5);
	  CHECK_ERROR (fread (&Rfa_bins_used, sizeof(int), 1, fil), 1);

          /* Load the measurement parameters */
          CHECK_ERROR (fread (Mstate, sizeof(double), 50, fil), 50);
          CHECK_ERROR (fread (&MUut,  sizeof(DEVICE), 1, fil), 1);
          CHECK_ERROR (fread (&MRef,  sizeof(DEVICE), 1, fil), 1);
          CHECK_ERROR (fread (&MExt,  sizeof(DEVICE), 1, fil), 1);
          CHECK_ERROR (fread (&MCnv,  sizeof(DEVICE), 1, fil), 1);

          /* Load Spur Accy Cal Tones (Usually contains 1e300) */
          //CHECK_ERROR (fread (Caltones, sizeof(double), 12, fil), 12);
          //revisit: no error check here because THEOR0.RES reads 0.
          fread (Caltones, sizeof(double), 12, fil);

     } while (0);
     return no_error;
}
/***************************************************************************/
//
// generate_freq_amplitude_pairs()
//
//	 ff1 -> 1st fft segment used.
//	 ff2 -> last fft segment used.
//	 rf1 -> 1st rf analyzer segment used.
//	 rf2 -> last rf analyzer segment used.
//
/**************************************************************************/
void generate_freq_ampl_pairs(int ff1, int ff2, int rf1, int rf2, char *output_file_name)
{
     FILE *output_file;
     int j;
     int seg;
     int bin, first_bin, last_bin;
     double  startf, lastf, resol;
     double freq, ampl;
     char title[81];
     int l;

     printf("Opening output file ...\n");

     if ((output_file = fopen(output_file_name, "w")) == NULL)
     {
       fprintf(stderr, "UNABLE TO OPEN %s\n", *output_file_name);
       exit(1);
     }

     printf("Generating ASCII translation ...\n");

     strcpy (title, Gtitle);
     l = strlen (title) - 1;

     while (l >= 0 && title [l] == ' ')
     {
	  title [l] = '\0';
	  l--;
     }

     fprintf(output_file, "GRAPH TITLE: %s\n", title);
     fprintf(output_file, "CARRIER FREQUENCY: %15.3f Hz.\n",Carrier);
     fprintf(output_file, "START DATE:  %d/%d/%d\n",Starttime.mo,Starttime.day,Starttime.yr);
     fprintf(output_file, "STOP  DATE:  %d/%d/%d\n",Stoptime.mo,Stoptime.day,Stoptime.yr);
     fprintf(output_file, "START TIME:  %d:%d:%d\n",Starttime.hr,Starttime.min,Starttime.sec);
     fprintf(output_file, "STOP  TIME:  %d:%d:%d\n",Stoptime.hr,Stoptime.min,Stoptime.sec);

     for (seg = ff1; seg <= ff2; seg++)
     {
	if ((seg >=0) &&(Fft_tags [seg] [0] < 1e300 ))  // if valid data here
	{
	  startf    = Fft_tags [seg] [2]; // 1st measured freq
	  lastf     = Fft_tags [seg] [3]; // last measured freq
	  resol     = (lastf - startf) / 400;
	  first_bin = Fft_tags [seg] [0];
	  last_bin  = Fft_tags [seg] [1];

	  for(bin = first_bin; bin <= last_bin; bin++)
	    {
	      freq = startf + (bin * resol);
	      if (unscale_trace_data(Fft_plot [seg] [bin], &ampl))
		fprintf(output_file, "%6.3f, %5.2f, ***SPUR***\n", freq, ampl);
	      else
		fprintf(output_file, "%6.3f, %5.2f\n", freq, ampl);
	    }
	}
     }

     for (seg = rf1; seg <= rf2; seg++)
     {
       if ((seg >= 0) && (Hf_tags [seg] [0] < 1e300 ))  // if valid data here
       {
	  startf    = Hf_tags [seg] [2];// 1st freq measured
	  lastf     = Hf_tags [seg] [3];// last freq measured
	  resol     = (lastf - startf) / Rfa_bins_used;
	  first_bin = Hf_tags [seg] [0]; // 1st bin to plot
	  last_bin  = Hf_tags [seg] [1]; // last bin to plot

	  for(bin = first_bin; bin <= last_bin; bin++)
	    {
	      freq = startf + (bin * resol);
	      if(unscale_trace_data(Hf_plot [seg] [bin], &ampl))
		fprintf(output_file, "%11.3f, %5.2f, ***SPUR***\n", freq, ampl);
	      else
		fprintf(output_file, "%11.3f, %5.2f\n", freq, ampl);
	    }
       }
     }
     fclose(output_file);
}
/***************************************************************************/
//
// unscale_trace_data()
//
// This function decodes the 3048A storage format and generates the
// equivalent dbc value.
//
// Data marked as a spur by 3048A are represented by positive integers.
// (Also, spur data are not normalized to a 1 Hz BW but rather the actual
// measurement bandwith.)
//
// Noise data are represented by negative integers.
// The scaling accomodates a range of +/- 300.01 dbc.
//
/**************************************************************************/
BOOLEAN unscale_trace_data(double i_ampl, double *dbc)
{
  if (i_ampl< 0)
  {
    *dbc = ((i_ampl + 15001.0) / 50.0);
    return FALSE; // not a spur
  }
  else
  {
    *dbc = ((i_ampl - 15001.0) / 50.0);
    return TRUE; // this is a spur
  }
}
/***************************************************************************/
FILE *open_binary_file(char *b_filename)
{
   FILE *fptr;

   /****************************************************************/
   /*                                                              */
   /*  NOTE: The .RES file was stored by 3048A as a binary file.   */
   /*        The 'rb' specifier in the call to fopen is required.  */
   /*                                                              */
   /* **************************************************************/

   if ((fptr = fopen(b_filename, "rb")) == NULL)
   {
     fprintf(stderr,"UNABLE TO OPEN %s\n", *b_filename);
     exit(1);
   }

   return fptr;
}